/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

[scriptable, uuid(C1B1F426-7E83-4759-9F88-0E1B17F49366)]
interface nsIByteRangeRequest : nsISupports
{
    /** 
     * Returns true IFF this request is a byte range request, otherwise it
     * returns false (This is effectively the same as checking to see if 
     * |startRequest| is zero and |endRange| is the content length.)
     */
    readonly attribute boolean isByteRangeRequest;

    /** 
     * Absolute start position in remote file for this request.
     */
    readonly attribute long long startRange;

    /**
     * Absolute end postion in remote file for this request
     */
    readonly attribute long long endRange;
};
