/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface imgIRequest;

%{C++
#include "nsRect.h"
%}

[ptr] native nsIntRect(nsIntRect);

[scriptable, builtinclass, uuid(ac65c702-7771-4f6d-b18b-1c7d806ce3c1)]
interface imgINotificationObserver : nsISupports
{
  const long SIZE_AVAILABLE = 1;
  const long FRAME_UPDATE = 2;
  const long FRAME_COMPLETE = 3;
  const long LOAD_COMPLETE = 4;
  const long DECODE_COMPLETE = 5;
  const long DISCARD = 6;
  const long UNLOCKED_DRAW = 7;
  const long IS_ANIMATED = 8;

  [noscript] void notify(in imgIRequest aProxy, in long aType, [const] in nsIntRect aRect);
};
