/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMEvent.idl"

interface nsIDOMCSSStyleSheet;

[builtinclass, uuid(f94c6a47-58f0-4996-b1db-3ea1d14c4ddd)]
interface nsIDOMStyleSheetChangeEvent : nsIDOMEvent
{
  readonly attribute nsIDOMCSSStyleSheet stylesheet;
  readonly attribute boolean documentSheet;
  [noscript] void initStyleSheetChangeEvent(in DOMString aTypeArg,
                                            in boolean aCanBubbleArg,
                                            in boolean aCancelableArg,
                                            in nsIDOMCSSStyleSheet aStyleSheet,
                                            in boolean aDocumentSheet);
};
